@@ -1,14 +1,13 @@
 // Process server for soft ioc
 // David H. Thompson 8/29/2003
-// Ralph Lange <ralph.lange@gmx.de> 2007-2016
+// Ralph Lange <ralph.lange@gmx.de> 2007-2017
 // GNU Public License (GPLv3) applies - see www.gnu.org
 
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h> 
-#include <sys/socket.h> 
-#include <netinet/in.h> 
+#include <sys/types.h>
+#include <sys/socket.h>
 #include <errno.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -19,6 +18,10 @@
 #include "processClass.h"
 #include "libtelnet.h"
 
+// Wrapper to ignore return values
+template<typename T>
+inline void ignore_result(T /* unused result */) {}
+
 static const telnet_telopt_t my_telopts[] = {
   { TELNET_TELOPT_ECHO,      TELNET_WILL,           0 },
   { TELNET_TELOPT_LINEMODE,            0, TELNET_DO   },
@@ -59,9 +62,12 @@ connectionItem * clientFactory(int socketIn, bool readonly)
 
 clientItem::~clientItem()
 {
-    if ( _fd >= 0 ) close( _fd );
+    if (_fd >= 0) {
+        shutdown(_fd, SHUT_RDWR);
+        close(_fd);
+    }
     if (_telnet) telnet_free(_telnet);
-    PRINTF("~clientItem()\n");
+    PRINTF("~clientItem(); handle %d closed\n", _fd);
     if (_readonly) _loggers--;
     else _users--;
 }
@@ -134,17 +140,17 @@ clientItem::clientItem(int socketIn, bool readonly) :
         _loggers++;
     } else {                    // Regular (user) client
         _users++;
-        write( _fd, greeting1, strlen(greeting1) );
-        write( _fd, greeting2, strlen(greeting2) );
+        ignore_result( write(_fd, greeting1, strlen(greeting1)) );
+        ignore_result( write(_fd, greeting2, strlen(greeting2)) );
     }
 
-    write( _fd, infoMessage1, strlen(infoMessage1) );
-    write( _fd, infoMessage2, strlen(infoMessage2) );
-    write( _fd, buf1, strlen(buf1) );
+    ignore_result( write(_fd, infoMessage1, strlen(infoMessage1)) );
+    ignore_result( write( _fd, infoMessage2, strlen(infoMessage2)) );
+    ignore_result( write( _fd, buf1, strlen(buf1)) );
     if ( ! _readonly )
-        write( _fd, buf2, strlen(buf2) );
+        ignore_result( write(_fd, buf2, strlen(buf2)) );
     if ( ! processClass::exists() )
-        write( _fd, infoMessage3, strlen(infoMessage3) );
+        ignore_result( write(_fd, infoMessage3, strlen(infoMessage3)) );
 
     _telnet = telnet_init(my_telopts, telnet_eh, 0, this);
 
