@@ -1,10 +1,14 @@
 // Process server for soft ioc
 // David H. Thompson 8/29/2003
-// Ralph Lange <ralph.lange@gmx.de> 2007-2016
+// Ralph Lange <ralph.lange@gmx.de> 2007-2017
 // Ambroz Bizjak 02/29/2016
 // Freddie Akeroyd 2016
+// Michael Davidsaver 2017
 // GNU Public License (GPLv3) applies - see www.gnu.org
 
+#include <vector>
+#include <string>
+#include <fstream>
 
 #include <stdio.h>
 #include <assert.h>
@@ -28,6 +32,10 @@
 
 #include "procServ.h"
 
+// Wrapper to ignore return values
+template<typename T>
+inline void ignore_result(T /* unused result */) {}
+
 #ifdef ALLOW_FROM_ANYWHERE
 const bool enableAllow = true;   // Enable --allow option
 #else
@@ -43,6 +51,7 @@ bool   waitForManualStart = false;  // Waits for telnet cmd to manually start ch
 volatile bool shutdownServer = false;   // To keep the server from shutting down
 bool   quiet = false;            // Suppress info output (server)
 bool   setCoreSize = false;      // Set core size for child
+bool   singleEndpointStyle = true;  // Compatibility style: first non-option is endpoint
 char   *procservName;            // The name of this beast (server)
 char   *childName;               // The name of that beast (child)
 char   *childExec;               // Exec to run as child
@@ -63,10 +72,10 @@ time_t holdoffTime = 15;         // Holdoff time between child restarts (in seco
 pid_t  procservPid;              // PID of server (daemon if not in debug mode)
 char   *pidFile;                 // File name for server PID
 char   defaultpidFile[] = "pid.txt";  // default
-char   *timeFormat;              // Time format string
+const char *timeFormat = "%c";       // Time format string
 char   defaulttimeFormat[] = "%c";    // default
 bool   stampLog = false;         // Prefix log lines with time stamp
-char   *stampFormat;             // Log time stamp format string
+const char *stampFormat;             // Log time stamp format string
 
 const size_t INFO1LEN = 512;
 const size_t INFO2LEN = 128;
@@ -78,7 +87,7 @@ char   infoMessage3[INFO3LEN];   // Sign on message: available server commands
 
 char   *logFile = NULL;          // File name for log
 int    logFileFD=-1;             // FD for log file
-int    logPort;                  // Port no. for logger connections
+char  *logPort;                  // address for logger connections
 int    debugFD=-1;               // FD for debug output
 
 #define MAX_CONNECTIONS 64
@@ -92,6 +101,7 @@ void forkAndGo();
 // Checks the command file (existence and access rights)
 int checkCommandFile(const char *command);
 void openLogFile();
+void writeInfoFile(const std::string& infofile);
 void ttySetCharNoEcho(bool save);
 
 // Signal handlers
@@ -137,40 +147,46 @@ char getOptionChar ( const char* buf )
 
 void printUsage()
 {
-    printf("Usage: %s [options] <port> <command args ...>    (-h for help)\n",
-           procservName);
+    printf("Usage: %s [options] -P <endpoint>... <command args ...>    (-h for help)\n"
+           "       %s [options] <endpoint> <command args ...>\n",
+           procservName, procservName);
 }
 
 void printHelp()
 {
     printUsage();
-    printf("<port>              use telnet <port> for command connections\n"
-           "<command args ...>  command line to start child process\n"
+    printf("<endpoint>:          endpoint to use for control connections\n"
+           "    <port>           TCP <port> on local/all interfaces (see --allow/--restrict)\n"
+           "    <iface>:<port>   TCP <port> on specific IP <iface> (numeric)\n"
+           "    unix:<path>      UNIX domain socket at <path> (@... for abstract)\n"
+           "<command args ...>   command line to start child process\n"
            "Options:\n"
-           "    --allow             allow control connections from anywhere\n"
-           "    --autorestartcmd    command to toggle auto restart flag (^ for ctrl)\n"
-           "    --coresize <n>      set maximum core size for child to <n>\n"
-           " -c --chdir <dir>       change directory to <dir> before starting child\n"
-           " -d --debug             debug mode (keeps child in foreground)\n"
-           " -e --exec <str>        specify child executable (default: arg0 of <command>)\n"
-           " -f --foreground        keep child in foreground (interactive)\n"
-           " -h --help              print this message\n"
-           "    --holdoff <n>       set holdoff time between child restarts\n"
-           " -i --ignore <str>      ignore all chars in <str> (^ for ctrl)\n"
-           " -k --killcmd <str>     command to kill (reboot) the child (^ for ctrl)\n"
-           "    --killsig <n>       signal to send to child when killing\n"
-           " -l --logport <n>       allow log connections through telnet port <n>\n"
-           " -L --logfile <file>    write log to <file>\n"
-           "    --logstamp [<str>]  prefix log lines with timestamp [strftime format]\n"
-           " -n --name <str>        set child's name (default: arg0 of <command>)\n"
-           "    --noautorestart     do not restart child on exit by default\n"
-           " -p --pidfile <str>     name of PID file (for server PID)\n"
-           " -q --quiet             suppress informational output (server)\n"
-           "    --restrict          restrict log connections to localhost\n"
-           "    --timefmt <str>     set time format (strftime) to <str>\n"
-           " -V --version           print program version\n"
-           " -w --wait              wait for telnet cmd to manually start child\n"
-           " -x --logoutcmd <str>   command to logout client connection (^ for ctrl)\n"
+           "    --allow               allow control connections from anywhere\n"
+           "    --autorestartcmd      command to toggle auto restart flag (^ for ctrl)\n"
+           "    --coresize <n>        set maximum core size for child to <n>\n"
+           " -c --chdir <dir>         change directory to <dir> before starting child\n"
+           " -d --debug               debug mode (keeps child in foreground)\n"
+           " -e --exec <str>          specify child executable (default: arg0 of <command>)\n"
+           " -f --foreground          keep child in foreground (interactive)\n"
+           " -h --help                print this message\n"
+           "    --holdoff <n>         set holdoff time [sec] between child restarts\n"
+           " -i --ignore <str>        ignore all chars in <str> (^ for ctrl)\n"
+           " -I --info-file <file>    write instance information to this file\n"
+           " -k --killcmd <str>       command to kill (reboot) the child (^ for ctrl)\n"
+           "    --killsig <n>         signal to send to child when killing\n"
+           " -l --logport <endpoint>  allow log connections through telnet <endpoint>\n"
+           " -L --logfile <file>      write log to <file>, '-' logs to stdout\n"
+           "    --logstamp [<str>]    prefix log lines with timestamp [strftime format]\n"
+           " -n --name <str>          set child's name (default: arg0 of <command>)\n"
+           "    --noautorestart       do not restart child on exit by default\n"
+           " -p --pidfile <str>       write PID file (for server PID)\n"
+           " -P --port <endpoint>     allow control connections through telnet <endpoint>\n"
+           " -q --quiet               suppress informational output (server)\n"
+           "    --restrict            restrict log access to connections from localhost\n"
+           "    --timefmt <str>       set time format (strftime) to <str>\n"
+           " -V --version             print program version\n"
+           " -w --wait                wait for cmd on control connection to start child\n"
+           " -x --logoutcmd <str>     command to logout client connection (^ for ctrl)\n"
         );
 }
 
@@ -184,11 +200,12 @@ int main(int argc,char * argv[])
     int c;
     unsigned int i, j;
     int k;
-    int ctlPort = 0;
+    std::vector<std::string> ctlSpecs;
     char *command;
-    bool wrongOption = false;
+    bool bailout = false;
     const size_t BUFLEN = 512;
     char buff[BUFLEN];
+    std::string infofile;
 
     time(&procServStart);             // remember start time
     procservName = argv[0];
@@ -214,6 +231,7 @@ int main(int argc,char * argv[])
             {"help",           no_argument,       0, 'h'},
             {"holdoff",        required_argument, 0, 'H'},
             {"ignore",         required_argument, 0, 'i'},
+            {"info-file",      required_argument, 0, 'I'},
             {"killcmd",        required_argument, 0, 'k'},
             {"killsig",        required_argument, 0, 'K'},
             {"logport",        required_argument, 0, 'l'},
@@ -222,6 +240,7 @@ int main(int argc,char * argv[])
             {"name",           required_argument, 0, 'n'},
             {"noautorestart",  no_argument,       0, 'N'},
             {"pidfile",        required_argument, 0, 'p'},
+            {"port",           required_argument, 0, 'P'},
             {"quiet",          no_argument,       0, 'q'},
             {"restrict",       no_argument,       0, 'R'},
             {"timefmt",        required_argument, 0, 'F'},
@@ -234,7 +253,7 @@ int main(int argc,char * argv[])
         /* getopt_long stores the option index here. */
         int option_index = 0;
 
-        c = getopt_long (argc, argv, "+c:de:fhi:k:l:L:n:p:qVwx:",
+        c = getopt_long (argc, argv, "+c:de:fhi:I:k:l:L:n:p:P:qVwx:",
                          long_options, &option_index);
 
         /* Detect the end of the options. */
@@ -308,6 +327,10 @@ int main(int argc,char * argv[])
             }
             break;
 
+        case 'I':                                 // Info file
+            infofile = optarg;
+            break;
+
         case 'k':                                 // Kill command
             killChar = getOptionChar ( optarg );
             break;
@@ -324,13 +347,7 @@ int main(int argc,char * argv[])
             break;
 
         case 'l':                                 // Log port
-            logPort = abs ( atoi( optarg ) );
-            if ( logPort < 1024 ) {
-                fprintf( stderr,
-                         "%s: invalid log port %d (<1024) - disabling log port\n",
-                         procservName, logPort );
-                logPort = 0;
-            }
+            logPort = strdup ( optarg );
             break;
 
         case 'L':                                 // Log file
@@ -353,6 +370,11 @@ int main(int argc,char * argv[])
             pidFile = strdup( optarg );
             break;
 
+        case 'P':                                 // Control port
+            ctlSpecs.push_back(optarg);
+            singleEndpointStyle = false;
+            break;
+
         case 'q':                                 // Quiet
             quiet = true;
             break;
@@ -375,7 +397,7 @@ int main(int argc,char * argv[])
 
         case '?':                                 // Error
             /* getopt_long already printed an error message */
-            wrongOption = true;
+            bailout = true;
             break;
 
         default:
@@ -383,15 +405,14 @@ int main(int argc,char * argv[])
         }
     }
 
-    if ( (argc-optind) < 2 )
-    {
-        fprintf( stderr, "%s: missing argument\n", procservName );
+    if ((argc - optind) < (singleEndpointStyle ? 2 : 1)) {
+        fprintf(stderr, "%s: missing argument\n", procservName);
+        bailout = true;
     }
 
-    if ( wrongOption || (argc-optind) < 2 )
-    {
-	printUsage();
-	exit(1);
+    if (bailout) {
+        printUsage();
+        exit(1);
     }
 
     // Single command characters should be ignored, too
@@ -416,28 +437,23 @@ int main(int argc,char * argv[])
     }
     strncat(infoMessage3, NL, INFO3LEN-strlen(infoMessage3)-1);
 
-    ctlPort = atoi(argv[optind]);
-    command = argv[optind+1];
+    if (singleEndpointStyle) {
+        ctlSpecs.push_back(argv[optind++]);
+    }
+    command = argv[optind];
 
     if (childName == NULL) childName = command;
-    childArgv = argv + optind;
+    childArgv = argv + optind - 1;
     if (childExec == NULL) {
         childArgv++;
         childExec = command;
     }
 
-    if ( ctlPort < 1024 )
-    {
-        fprintf( stderr,
-                 "%s: invalid control port %d (<1024)\n",
-                 procservName, ctlPort );
-        exit(1);
-    }
-
     if (!stampFormat) {
-        stampFormat = (char*) calloc(strlen(timeFormat)+4, 1);
         if (stampFormat) {
-            sprintf(stampFormat, "[%s] ", timeFormat);
+            char *tmp = (char*) calloc(strlen(timeFormat)+4, 1);
+            sprintf(tmp, "[%s] ", timeFormat);
+            stampFormat = tmp;
         } else {
             stampFormat = timeFormat;
         }
@@ -486,15 +502,17 @@ int main(int argc,char * argv[])
     PRINTF("Creating control listener\n");
     try
     {
-	connectionItem *acceptItem = acceptFactory( ctlPort, ctlPortLocal );
-	AddConnection(acceptItem);
+        for(size_t i=0; i<ctlSpecs.size(); i++) {
+            connectionItem *acceptItem = acceptFactory( ctlSpecs[i].c_str(), ctlPortLocal );
+            AddConnection(acceptItem);
+        }
     }
     catch (int error)
     {
-	perror("Caught an exception creating the initial control telnet port");
-	fprintf(stderr, "%s: Exiting with error code: %d\n",
+        perror("Caught an exception creating the initial control telnet port");
+        fprintf(stderr, "%s: Exiting with error code: %d\n",
                 procservName, error);
-	exit(error);
+        exit(error);
     }
 
     if ( logPort ) {
@@ -528,7 +546,11 @@ int main(int argc,char * argv[])
         debugFD = 1;          // Enable debug messages
     }
 
-    if (inFgMode) {
+    if (!infofile.empty()) {
+        writeInfoFile(infofile);
+    }
+
+    if (inFgMode && !(logFile && strcmp(logFile, "-")==0)) {
         ttySetCharNoEcho(true);
         AddConnection(clientFactory(0));
     }
@@ -626,6 +648,21 @@ int main(int argc,char * argv[])
         }
     }
     ttySetCharNoEcho(false);
+
+    PRINTF("Close sockets\n");
+
+    while(connectionItem::head) {
+        connectionItem *p = connectionItem::head;
+        connectionItem::head = p->next;
+        delete p;
+    }
+
+    PRINTF("Cleanup pid and info files\n");
+
+    if(!infofile.empty())
+        unlink(infofile.c_str());
+    if(pidFile)
+        unlink(pidFile);
 }
 
 // Connection items call this to send messages to others
@@ -659,22 +696,22 @@ void SendToAll(const char * message,
                 int i = 0, j = 0;
                 for (i = 0; i < count; ++i) {
                     if (!log_stamp_sent) {
-                        write(logFileFD, stamp, len);
+                        ignore_result( write(logFileFD, stamp, len) );
                         log_stamp_sent = true;
                     }
                     if (message[i] == '\n') {
-                        write(logFileFD, message+j, i-j+1);
+                        ignore_result( write(logFileFD, message+j, i-j+1) );
                         j = i + 1;
                         log_stamp_sent = false;
                     }
                 }
-                write(logFileFD, message+j, count-j);  // finish off rest of line with no newline at end
+                ignore_result( write(logFileFD, message+j, count-j) );  // finish off rest of line with no newline at end
             } else {
-                write(logFileFD, message, count);
+                ignore_result( write(logFileFD, message, count) );
             }
             fsync(logFileFD);
         }
-        if (inFgMode == false && debugFD > 0) write(debugFD, message, count);
+        if (inFgMode == false && debugFD > 0) ignore_result( write(debugFD, message, count) );
     }
 
     while (p) {
@@ -820,7 +857,7 @@ void forkAndGo()
         fh = open(buf, O_RDWR);
         if (fh < 0) { perror(buf); exit(-1); }
         close(0); close(1); close(2);
-        dup(fh); dup(fh); dup(fh);
+        ignore_result( dup(fh) ); ignore_result( dup(fh) ); ignore_result( dup(fh) );
         close(fh);
 
         // Make sure we are not attached to a terminal
@@ -877,9 +914,12 @@ int checkCommandFile(const char * command)
 
 void openLogFile()
 {
-    if (-1 != logFileFD) {
+    if (-1 != logFileFD && 1 != logFileFD) {
         close(logFileFD);
     }
+    if (logFile && strcmp(logFile, "-")==0) {
+        logFileFD = 1;
+    } else
     if (logFile) {
         logFileFD = open(logFile, O_CREAT|O_WRONLY|O_APPEND, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
         if (-1 == logFileFD) {         // Don't stop here - just go without
@@ -892,11 +932,21 @@ void openLogFile()
     }
 }
 
+void writeInfoFile(const std::string& infofile)
+{
+    std::ofstream info(infofile.c_str());
+    info<<"pid:"<<getpid()<<"\n";
+    for(connectionItem *it = connectionItem::head; it; it=it->next)
+        it->writeAddress(info);
+}
+
 void ttySetCharNoEcho(bool set) {
     static struct termios org_mode;
     static struct termios mode;
     static bool saved = false;
 
+    if(isatty(0)!=1) return;
+
     if (set && !saved) {
         tcgetattr(0, &mode);
         org_mode = mode;
