@@ -1,12 +1,15 @@
 // Process server for soft ioc
 // David H. Thompson 8/29/2003
 // Ralph Lange <ralph.lange@gmx.de> 2007-2016
+// Michael Davidsaver 2017
 // GNU Public License (GPLv3) applies - see www.gnu.org
 
 
 #ifndef procServH
 #define procServH
 
+#include <ostream>
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -17,6 +20,12 @@
 #include <stdio.h>
 #include <time.h>
 
+/* whether to enable UNIX domain sockets */
+#ifdef __unix__
+#include <sys/un.h>
+# define USOCKS
+#endif
+
 #ifndef PRINTF
 #define PRINTF if (inDebugMode) printf
 #endif
@@ -32,7 +41,7 @@ extern bool   setCoreSize;
 extern char   *procservName;
 extern char   *childName;
 extern char   *ignChars;
-extern char   *timeFormat;
+extern const char   *timeFormat;
 extern char   killChar;
 extern char   toggleRestartChar;
 extern char   restartChar;
@@ -86,7 +95,7 @@ connectionItem * clientFactory(int ioSocket, bool readonly=false);
 // service and calls clientFactory when clients are accepted
 // local: restrict to localhost (127.0.0.1)
 // readonly: discard any input from the client
-connectionItem * acceptFactory( int port, bool local=true, bool readonly=false );
+connectionItem * acceptFactory( const char *spec, bool local=true, bool readonly=false );
 
 extern connectionItem * processItem; // Set if it exists
  
@@ -116,6 +125,7 @@ public:
     virtual bool isProcess() const { return false; }
     virtual bool isLogger() const { return _readonly; }
 
+    virtual void writeAddress(std::ostream& fp) {}
 protected:
     connectionItem ( int fd = -1, bool readonly = false );
     int _fd;                 // File descriptor of this connection
